@@ -36,6 +36,8 @@
 #include "../interfaces/ipartcontroller.h"
 #include "breakpoint.h"
 #include <KConfigGroup>
+#include <QAction>
+#include <QMenu>
 
 #define IF_DEBUG(x)
 
@@ -46,9 +48,6 @@ BreakpointModel::BreakpointModel(QObject* parent)
     : QAbstractTableModel(parent),
       m_dontUpdateMarks(false)
 {
-    connect(this, SIGNAL(rowsInserted(const QModelIndex &, int, int)), SLOT(save()));
-    connect(this, SIGNAL(rowsRemoved(const QModelIndex &, int, int)), SLOT(save()));
-    connect(this, SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)), SLOT(save()));
     connect(this, SIGNAL(dataChanged(QModelIndex,QModelIndex)), SLOT(updateMarks()));
 
     if (KDevelop::ICore::self()->partController()) { //TODO remove if
@@ -71,7 +70,10 @@ BreakpointModel::BreakpointModel(QObject* parent)
     load();
 }
 
-BreakpointModel::~BreakpointModel() {
+BreakpointModel::~BreakpointModel()
+{
+    save();
+
     qDeleteAll(m_breakpoints);
 }
 
@@ -89,7 +91,6 @@ void BreakpointModel::slotPartAdded(KParts::Part* part)
         iface->setMarkPixmap((MarkInterface::MarkTypes)ReachedBreakpointMark, *reachedBreakpointPixmap());
         iface->setMarkPixmap((MarkInterface::MarkTypes)DisabledBreakpointMark, *disabledBreakpointPixmap());
         iface->setEditableMarks( MarkInterface::Bookmark | BreakpointMark );
-        
         updateMarks();
     }
 }
@@ -108,9 +109,47 @@ void BreakpointModel::textDocumentCreated(KDevelop::IDocument* doc)
                  SLOT(markChanged(KTextEditor::Document*,
                                  KTextEditor::Mark,
                                   KTextEditor::MarkInterface::MarkChangeAction)));
+        connect(doc->textDocument(), SIGNAL(markContextMenuRequested(KTextEditor::Document*, KTextEditor::Mark, QPoint, bool&)),
+                SLOT(markContextMenuRequested(KTextEditor::Document*, KTextEditor::Mark, QPoint, bool&)));
     }
 }
 
+void BreakpointModel::markContextMenuRequested(Document* document, Mark mark, const QPoint &pos, bool& handled)
+{
+
+    int type = mark.type;
+    kDebug() << type;
+
+    /* Is this a breakpoint mark, to begin with? */
+    if (!(type & AllBreakpointMarks)) return;
+
+    Breakpoint *b = breakpoint(document->url(), mark.line);
+    Q_ASSERT(b);
+    if (!b) return;
+
+    QMenu menu;
+    QAction deleteAction(KIcon("edit-delete"), i18n("&Delete Breakpoint"), 0);
+    QAction disableAction(KIcon("dialog-cancel"), i18n("&Disable Breakpoint"), 0);
+    QAction enableAction(KIcon("dialog-ok-apply"), i18n("&Enable Breakpoint"), 0);
+    menu.addAction(&deleteAction);
+    if (b->enabled()) {
+        menu.addAction(&disableAction);
+    } else {
+        menu.addAction(&enableAction);
+    }
+    QAction *a = menu.exec(pos);
+    if (a == &deleteAction) {
+        b->setDeleted();
+    } else if (a == &disableAction) {
+        b->setData(Breakpoint::EnableColumn, Qt::Unchecked);
+    } else if (a == &enableAction) {
+        b->setData(Breakpoint::EnableColumn, Qt::Checked);
+    }
+
+    handled = true;
+}
+
+
 QVariant 
 BreakpointModel::headerData(int section, Qt::Orientation orientation,
                                  int role) const
@@ -169,16 +208,15 @@ QModelIndex BreakpointModel::breakpointIndex(KDevelop::Breakpoint* b, int column
 
 bool KDevelop::BreakpointModel::removeRows(int row, int count, const QModelIndex& parent)
 {
-    if (row + count > m_breakpoints.count()) {
-        count = m_breakpoints.count() - row;
-        if (count <= 0) return false;
-    }
+    if (count < 1 || (row < 0) || (row + count) > rowCount(parent))
+        return false;
+
     beginRemoveRows(parent, row, row+count-1);
     for (int i=0; i < count; ++i) {
         Breakpoint *b = m_breakpoints.at(row);
         m_breakpoints.removeAt(row);
         IF_DEBUG ( kDebug() << m_breakpoints; )
-        if (!b->deleted()) b->setDeleted();
+        b->setDeleted();
         emit breakpointDeleted(b);
     }
     endRemoveRows();
@@ -189,7 +227,7 @@ bool KDevelop::BreakpointModel::removeRows(int row, int count, const QModelIndex
 int KDevelop::BreakpointModel::rowCount(const QModelIndex& parent) const
 {
     if (!parent.isValid()) {
-        return m_breakpoints.count() + 1;
+        return m_breakpoints.count();
     }
     return 0;
 }
@@ -202,25 +240,6 @@ int KDevelop::BreakpointModel::columnCount(const QModelIndex& parent) const
 
 QVariant BreakpointModel::data(const QModelIndex& index, int role) const
 {
-    if (!index.parent().isValid() && index.row() == m_breakpoints.count()) {
-        if (index.column() != Breakpoint::LocationColumn) {
-            if (role == Qt::DisplayRole) {
-                return QString();
-            } else {
-                return QVariant();
-            }
-        }
-
-        if (role == Qt::DisplayRole)
-            return i18n("Double-click to create new code breakpoint");
-        if (role == Qt::ForegroundRole)
-            // FIXME: returning hardcoded gray is bad,
-            // but we don't have access to any widget, or pallette
-            // thereof, at this point.
-            return QColor(128, 128, 128);
-        if (role == Qt::EditRole)
-            return QString();
-    }
     if (!index.parent().isValid() && index.row() < m_breakpoints.count()) {
         return m_breakpoints.at(index.row())->data(index.column(), role);
     }
@@ -229,21 +248,10 @@ QVariant BreakpointModel::data(const QModelIndex& index, int role) const
 
 bool KDevelop::BreakpointModel::setData(const QModelIndex& index, const QVariant& value, int role)
 {
-    if (!index.parent().isValid() && index.row() == m_breakpoints.count()
-        && role == Qt::EditRole
-        && (index.column() == Breakpoint::LocationColumn || index.column() == Breakpoint::ConditionColumn)
-        && !value.toString().isEmpty())
-    {
-        /* Helper breakpoint becomes a real breakpoint only if user types
-        some real location.  */
-        addCodeBreakpoint(); //setData below is called
-    }
-
     if (!index.parent().isValid() && index.row() < m_breakpoints.count() && (role == Qt::EditRole || role == Qt::CheckStateRole)) {
         return m_breakpoints.at(index.row())->setData(index.column(), value);
     }
     return false;
-
 }
 
 void BreakpointModel::markChanged(
@@ -324,8 +332,12 @@ void BreakpointModel::toggleBreakpoint(const KUrl& url, const KTextEditor::Curso
 
 void BreakpointModel::reportChange(Breakpoint* breakpoint, Breakpoint::Column column)
 {
-    QModelIndex idx = breakpointIndex(breakpoint, column);
-    emit dataChanged(idx, idx);
+    // note: just a portion of Breakpoint::Column is displayed in this model!
+    if (column >= 0 && column < columnCount()) {
+        QModelIndex idx = breakpointIndex(breakpoint, column);
+        Q_ASSERT(idx.isValid()); // make sure we don't pass invalid indices to dataChanged()
+        emit dataChanged(idx, idx);
+    }
     emit breakpointChanged(breakpoint, column);
 }
 
@@ -334,7 +346,7 @@ uint BreakpointModel::breakpointType(Breakpoint *breakpoint)
     uint type = BreakpointMark;
     if (!breakpoint->enabled()) {
         type = DisabledBreakpointMark;
-    } else if (breakpoint->hitCount()) {
+    } else if (breakpoint->hitCount() > 0) {
         type = ReachedBreakpointMark;
     } else if (breakpoint->state() == Breakpoint::PendingState) {
         type = PendingBreakpointMark;
@@ -417,13 +429,16 @@ void BreakpointModel::load()
 {
     KConfigGroup breakpoints = KGlobal::config()->group("breakpoints");
     int count = breakpoints.readEntry("number", 0);
+    if (count == 0)
+        return;
+
+    beginInsertRows(QModelIndex(), 0, count);
     for (int i = 0; i < count; ++i) {
         if (!breakpoints.group(QString::number(i)).readEntry("kind", "").isEmpty()) {
-            Breakpoint *b = new Breakpoint(this, breakpoints.group(QString::number(i)));
-            m_breakpoints << b;
+            new Breakpoint(this, breakpoints.group(QString::number(i)));
         }
     }
-    reset();
+    endInsertRows();
 }
 
 void BreakpointModel::save()
@@ -436,6 +451,7 @@ void BreakpointModel::save()
         b->save(g);
         ++i;
     }
+    breakpoints.sync();
 }
 
 QList<Breakpoint*> KDevelop::BreakpointModel::breakpoints() const
@@ -453,7 +469,6 @@ Breakpoint* BreakpointModel::addCodeBreakpoint()
 {
     beginInsertRows(QModelIndex(), m_breakpoints.count(), m_breakpoints.count());
     Breakpoint* n = new Breakpoint(this, Breakpoint::CodeBreakpoint);
-    m_breakpoints << n;
     endInsertRows();
     return n;
 }
@@ -476,7 +491,6 @@ Breakpoint* BreakpointModel::addWatchpoint()
 {
     beginInsertRows(QModelIndex(), m_breakpoints.count(), m_breakpoints.count());
     Breakpoint* n = new Breakpoint(this, Breakpoint::WriteBreakpoint);
-    m_breakpoints << n;
     endInsertRows();
     return n;
 }
@@ -492,7 +506,6 @@ Breakpoint* BreakpointModel::addReadWatchpoint()
 {
     beginInsertRows(QModelIndex(), m_breakpoints.count(), m_breakpoints.count());
     Breakpoint* n = new Breakpoint(this, Breakpoint::ReadBreakpoint);
-    m_breakpoints << n;
     endInsertRows();
     return n;
 }
@@ -508,7 +521,6 @@ Breakpoint* BreakpointModel::addAccessWatchpoint()
 {
     beginInsertRows(QModelIndex(), m_breakpoints.count(), m_breakpoints.count());
     Breakpoint* n = new Breakpoint(this, Breakpoint::AccessBreakpoint);
-    m_breakpoints << n;
     endInsertRows();
     return n;
 }
@@ -521,6 +533,12 @@ Breakpoint* BreakpointModel::addAccessWatchpoint(const QString& expression)
     return n;
 }
 
+void BreakpointModel::registerBreakpoint(Breakpoint* breakpoint)
+{
+    Q_ASSERT(!m_breakpoints.contains(breakpoint));
+    m_breakpoints << breakpoint;
+}
+
 Breakpoint* BreakpointModel::breakpoint(const KUrl& url, int line) {
     foreach (Breakpoint *b, m_breakpoints) {
         if (b->url() == url && b->line() == line) {
