@@ -22,10 +22,8 @@
 #include <KDE/KSharedPtr>
 
 #include "../../languageexport.h"
-#include "../duchainpointer.h"
+#include "../indexeddeclaration.h"
 #include "navigationaction.h"
-#include "../types/structuretype.h"
-#include <util/ksharedobject.h>
 
 namespace KDevelop {
 
