@@ -31,6 +31,35 @@
 namespace KDevelop
 {
 
+FilteredItem FilteringStrategyUtils::match(const QList<ErrorFormat>& errorFormats, const QString& line)
+{
+    FilteredItem item(line);
+    foreach( const ErrorFormat& curErrFilter, errorFormats ) {
+        QRegExp regEx = curErrFilter.expression;
+        if( regEx.indexIn( line ) != -1 )
+        {
+            item.url = regEx.cap( curErrFilter.fileGroup );
+            item.lineNo = regEx.cap( curErrFilter.lineGroup ).toInt() - 1;
+            if(curErrFilter.columnGroup >= 0) {
+                item.columnNo = regEx.cap( curErrFilter.columnGroup ).toInt() - 1;
+            } else {
+                item.columnNo = 0;
+            }
+
+            QString txt = regEx.cap(curErrFilter.textGroup);
+
+            item.type = FilteredItem::ErrorItem;
+
+            // Make the item clickable if it comes with the necessary file & line number information
+            if (curErrFilter.fileGroup > 0 && curErrFilter.lineGroup > 0) {
+                item.isActivatable = true;
+            }
+            break;
+        }
+    }
+    return item;
+}
+
 /// --- No filter strategy ---
 
 NoFilterStrategy::NoFilterStrategy()
@@ -47,7 +76,6 @@ FilteredItem NoFilterStrategy::errorInLine(const QString& line)
     return FilteredItem( line );
 }
 
-
 /// --- Compiler error filter strategy ---
 
 /// Impl. of CompilerFilterStrategy.
@@ -65,88 +93,107 @@ struct CompilerFilterStrategyPrivate
     PositionMap m_positionInCurrentDirs;
 };
 
+namespace {
 // All the possible string that indicate an error if we via Regex have been able to
 // extract file and linenumber from a given outputline
+// TODO: This seems clumsy -- and requires another scan of the line.
+// Merge this information into ErrorFormat? --Kevin
 typedef QPair<QString, FilteredItem::FilteredOutputItemType> Indicator;
-static const QVector<Indicator> INDICATORS = QVector<Indicator>()
+const QVector<Indicator> INDICATORS {
     // ld
-    << Indicator("undefined reference", FilteredItem::ErrorItem)
-    << Indicator("undefined symbol", FilteredItem::ErrorItem)
-    << Indicator("ld: cannot find", FilteredItem::ErrorItem)
-    << Indicator("no such file", FilteredItem::ErrorItem)
+    Indicator("undefined reference", FilteredItem::ErrorItem),
+    Indicator("undefined symbol", FilteredItem::ErrorItem),
+    Indicator("ld: cannot find", FilteredItem::ErrorItem),
+    Indicator("no such file", FilteredItem::ErrorItem),
     // gcc
-    << Indicator("error", FilteredItem::ErrorItem)
+    Indicator("error", FilteredItem::ErrorItem),
     // generic
-    << Indicator("warning", FilteredItem::WarningItem)
-    << Indicator("info", FilteredItem::InformationItem)
-    << Indicator("note", FilteredItem::InformationItem);
+    Indicator("warning", FilteredItem::WarningItem),
+    Indicator("info", FilteredItem::InformationItem),
+    Indicator("note", FilteredItem::InformationItem),
+};
 
 // A list of filters for possible compiler, linker, and make errors
-const QList<ErrorFormat> ERROR_FILTERS = QList<ErrorFormat>()
+const QVector<ErrorFormat> ERROR_FILTERS {
     // GCC - another case, eg. for #include "pixmap.xpm" which does not exists
-    << ErrorFormat( "^([^:\t]+):([0-9]+):([0-9]+):([^0-9]+)", 1, 2, 4, 3 )
+    ErrorFormat( "^([^:\t]+):([0-9]+):([0-9]+):([^0-9]+)", 1, 2, 4, 3 ),
     // GCC
-    << ErrorFormat( "^([^:\t]+):([0-9]+):([^0-9]+)", 1, 2, 3 )
+    ErrorFormat( "^([^:\t]+):([0-9]+):([^0-9]+)", 1, 2, 3 ),
     // GCC
-    << ErrorFormat( "^(In file included from |[ ]+from )([^: \\t]+):([0-9]+)(:|,)(|[0-9]+)", 2, 3, 5 )
+    ErrorFormat( "^(In file included from |[ ]+from )([^: \\t]+):([0-9]+)(:|,)(|[0-9]+)", 2, 3, 5 ),
     // ICC
-    << ErrorFormat( "^([^: \\t]+)\\(([0-9]+)\\):([^0-9]+)", 1, 2, 3, "intel" )
+    ErrorFormat( "^([^: \\t]+)\\(([0-9]+)\\):([^0-9]+)", 1, 2, 3, "intel" ),
     //libtool link
-    << ErrorFormat( "^(libtool):( link):( warning): ", 0, 0, 0 )
+    ErrorFormat( "^(libtool):( link):( warning): ", 0, 0, 0 ),
     // make
-    << ErrorFormat( "No rule to make target", 0, 0, 0 )
+    ErrorFormat( "No rule to make target", 0, 0, 0 ),
     // cmake
-    << ErrorFormat( "^([^: \\t]+):([0-9]+):", 1, 2, 0, "cmake" )
+    ErrorFormat( "^([^: \\t]+):([0-9]+):", 1, 2, 0, "cmake" ),
     // cmake
-    << ErrorFormat( "CMake (Error|Warning) (|\\([a-zA-Z]+\\) )(in|at) ([^:]+):($|[0-9]+)", 4, 5, 1, "cmake" )
+    ErrorFormat( "CMake (Error|Warning) (|\\([a-zA-Z]+\\) )(in|at) ([^:]+):($|[0-9]+)", 4, 5, 1, "cmake" ),
+    // cmake/automoc
+    // example: AUTOMOC: error: /foo/bar.cpp The file includes (...),
+    // example: AUTOMOC: error: /foo/bar.cpp: The file includes (...)
+    // note: ':' after file name isn't always appended, see http://cmake.org/gitweb?p=cmake.git;a=commitdiff;h=317d8498aa02c9f486bf5071963bb2034777cdd6
+    // example: AUTOGEN: error: /foo/bar.cpp: The file includes (...)
+    // note: AUTOMOC got renamed to AUTOGEN at some point
+    ErrorFormat( "^(AUTOMOC|AUTOGEN): error: ([^:]+):? (The file .*)$", 2, 0, 0 ),
+    // via qt4_automoc
+    // example: automoc4: The file "/foo/bar.cpp" includes the moc file "bar1.moc", but ...
+    ErrorFormat( "^automoc4: The file \"([^\"]+)\" includes the moc file", 1, 0, 0 ),
     // Fortran
-    << ErrorFormat( "\"(.*)\", line ([0-9]+):(.*)", 1, 2, 3 )
+    ErrorFormat( "\"(.*)\", line ([0-9]+):(.*)", 1, 2, 3 ),
     // GFortran
-    << ErrorFormat( "^(.*):([0-9]+)\\.([0-9]+):(.*)", 1, 2, 4, "gfortran", 3 )
+    ErrorFormat( "^(.*):([0-9]+)\\.([0-9]+):(.*)", 1, 2, 4, "gfortran", 3 ),
     // Jade
-    << ErrorFormat( "^[a-zA-Z]+:([^: \t]+):([0-9]+):[0-9]+:[a-zA-Z]:(.*)", 1, 2, 3 )
+    ErrorFormat( "^[a-zA-Z]+:([^: \t]+):([0-9]+):[0-9]+:[a-zA-Z]:(.*)", 1, 2, 3 ),
     // ifort
-    << ErrorFormat( "^fortcom: (.*): (.*), line ([0-9]+):(.*)", 2, 3, 1, "intel" )
+    ErrorFormat( "^fortcom: (.*): (.*), line ([0-9]+):(.*)", 2, 3, 1, "intel" ),
     // PGI
-    << ErrorFormat( "PGF9(.*)-(.*)-(.*)-(.*) \\((.*): ([0-9]+)\\)", 5, 6, 4, "pgi" )
+    ErrorFormat( "PGF9(.*)-(.*)-(.*)-(.*) \\((.*): ([0-9]+)\\)", 5, 6, 4, "pgi" ),
     // PGI (2)
-    << ErrorFormat( "PGF9(.*)-(.*)-(.*)-Symbol, (.*) \\((.*)\\)", 5, 5, 4, "pgi" );
+    ErrorFormat( "PGF9(.*)-(.*)-(.*)-Symbol, (.*) \\((.*)\\)", 5, 5, 4, "pgi" ),
+};
 
 // A list of filters for possible compiler, linker, and make actions
-QList<ActionFormat> ACTION_FILTERS = QList<ActionFormat>()
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "compiling"), 1, 2, "(?:^|[^=])\\b(gcc|CC|cc|distcc|c\\+\\+|"
+const QVector<ActionFormat> ACTION_FILTERS {
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "compiling"), 1, 2, "(?:^|[^=])\\b(gcc|CC|cc|distcc|c\\+\\+|"
                      "g\\+\\+|clang|clang\\+\\+|mpicc|icc|icpc)\\s+.*-c.*[/ '\\\\]+(\\w+\\.(?:cpp|CPP|c|C|cxx|CXX|cs|"
-                     "java|hpf|f|F|f90|F90|f95|F95))")
+                     "java|hpf|f|F|f90|F90|f95|F95))"),
     //moc and uic
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "generating"), 1, 2, "/(moc|uic)\\b.*\\s-o\\s([^\\s;]+)")
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "generating"), 1, 2, "/(moc|uic)\\b.*\\s-o\\s([^\\s;]+)"),
     //libtool linking
-    << ActionFormat( I18N_NOOP2_NOSTRIP("Linking object files into a library or executable", "linking"),
-                     "libtool", "/bin/sh\\s.*libtool.*--mode=link\\s.*\\s-o\\s([^\\s;]+)", 1 )
+    ActionFormat( I18N_NOOP2_NOSTRIP("Linking object files into a library or executable", "linking"),
+                     "libtool", "/bin/sh\\s.*libtool.*--mode=link\\s.*\\s-o\\s([^\\s;]+)", 1 ),
     //unsermake
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "compiling"), 1, 1, "^compiling (.*)" )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "generating"), 1, 2, "^generating (.*)" )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("Linking object files into a library or executable",
-                     "linking"), 1, 2, "(gcc|cc|c\\+\\+|g\\+\\+|clang|clang\\+\\+|mpicc|icc|icpc)\\S* (?:\\S* )*-o ([^\\s;]+)")
-    << ActionFormat( I18N_NOOP2_NOSTRIP("Linking object files into a library or executable",
-                     "linking"), 1, 2, "^linking (.*)" )
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "compiling"), 1, 1, "^compiling (.*)" ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "generating"), 1, 2, "^generating (.*)" ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("Linking object files into a library or executable",
+                     "linking"), 1, 2, "(gcc|cc|c\\+\\+|g\\+\\+|clang|clang\\+\\+|mpicc|icc|icpc)\\S* (?:\\S* )*-o ([^\\s;]+)"),
+    ActionFormat( I18N_NOOP2_NOSTRIP("Linking object files into a library or executable",
+                     "linking"), 1, 2, "^linking (.*)" ),
     //cmake
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "built"), -1, 1, "\\[.+%\\] Built target (.*)" )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "compiling"), "cmake", "\\[.+%\\] Building .* object (.*)CMakeFiles/", 1 )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "generating"), -1, 1, "\\[.+%\\] Generating (.*)" )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("Linking object files into a library or executable",
-                     "linking"), -1, 1, "^Linking (.*)" )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "configuring"), "cmake", "(-- Configuring (done|incomplete)|-- Found|-- Adding|-- Enabling)", -1 )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "installing"), -1, 1, "-- Installing (.*)" )
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "built"), -1, 1, "\\[.+%\\] Built target (.*)" ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "compiling"), "cmake", "\\[.+%\\] Building .* object (.*)CMakeFiles/", 1 ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "generating"), -1, 1, "\\[.+%\\] Generating (.*)" ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("Linking object files into a library or executable",
+                     "linking"), -1, 1, "^Linking (.*)" ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "configuring"), "cmake", "(-- Configuring (done|incomplete)|-- Found|-- Adding|-- Enabling)", -1 ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "installing"), -1, 1, "-- Installing (.*)" ),
     //libtool install
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "creating"), "", "/(?:bin/sh\\s.*mkinstalldirs).*\\s([^\\s;]+)", 1 )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "installing"), "", "/(?:usr/bin/install|bin/sh\\s.*mkinstalldirs"
-                     "|bin/sh\\s.*libtool.*--mode=install).*\\s([^\\s;]+)", 1 )
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "creating"), "", "/(?:bin/sh\\s.*mkinstalldirs).*\\s([^\\s;]+)", 1 ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "installing"), "", "/(?:usr/bin/install|bin/sh\\s.*mkinstalldirs"
+                     "|bin/sh\\s.*libtool.*--mode=install).*\\s([^\\s;]+)", 1 ),
     //dcop
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "generating"), "dcopidl", "dcopidl .* > ([^\\s;]+)", 1 )
-    << ActionFormat( I18N_NOOP2_NOSTRIP("", "compiling"), "dcopidl2cpp", "dcopidl2cpp (?:\\S* )*([^\\s;]+)", 1 )
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "generating"), "dcopidl", "dcopidl .* > ([^\\s;]+)", 1 ),
+    ActionFormat( I18N_NOOP2_NOSTRIP("", "compiling"), "dcopidl2cpp", "dcopidl2cpp (?:\\S* )*([^\\s;]+)", 1 ),
     // match against Entering directory to update current build dir
-    << ActionFormat( "", "cd", "", "make\\[\\d+\\]: Entering directory (\\`|\\')(.+)'", 2);
+    ActionFormat( "", "cd", "", "make\\[\\d+\\]: Entering directory (\\`|\\')(.+)'", 2),
+    // waf and scons use the same basic convention as make
+    ActionFormat( "", "cd", "", "(Waf|scons): Entering directory (\\`|\\')(.+)'", 3)
+};
 
+}
 
 CompilerFilterStrategyPrivate::CompilerFilterStrategyPrivate(const KUrl& buildDir)
 : m_buildDir(buildDir)
@@ -200,7 +247,6 @@ void CompilerFilterStrategyPrivate::putDirAtEnd(const QString& dirNameToInsert)
     }
 }
 
-
 CompilerFilterStrategy::CompilerFilterStrategy(const KUrl& buildDir)
 : d(new CompilerFilterStrategyPrivate( buildDir ))
 {
@@ -284,8 +330,8 @@ FilteredItem CompilerFilterStrategy::errorInLine(const QString& line)
                 }
             }
 
-            // Make the item clickable if it comes with the necessary file & line number information
-            if (curErrFilter.fileGroup > 0 && curErrFilter.lineGroup > 0) {
+            // Make the item clickable if it comes with the necessary file information
+            if (item.url.isValid()) {
                 item.isActivatable = true;
                 if(item.type == FilteredItem::InvalidItem) {
                     // If there are no error indicators in the line
@@ -329,31 +375,39 @@ FilteredItem ScriptErrorFilterStrategy::actionInLine(const QString& line)
 
 FilteredItem ScriptErrorFilterStrategy::errorInLine(const QString& line)
 {
-    FilteredItem item(line);
-    foreach( const ErrorFormat& curErrFilter, SCRIPT_ERROR_FILTERS ) {
-        QRegExp regEx = curErrFilter.expression;
-        if( regEx.indexIn( line ) != -1 )
-        {
-            item.url = regEx.cap( curErrFilter.fileGroup );
-            item.lineNo = regEx.cap( curErrFilter.lineGroup ).toInt() - 1;
-            if(curErrFilter.columnGroup >= 0) {
-                item.columnNo = regEx.cap( curErrFilter.columnGroup ).toInt() - 1;
-            } else {
-                item.columnNo = 0;
-            }
+    return FilteringStrategyUtils::match(SCRIPT_ERROR_FILTERS, line);
+}
 
-            QString txt = regEx.cap(curErrFilter.textGroup);
+/// --- Native application error filter strategy ---
 
-            item.type = FilteredItem::ErrorItem;
+const QList<ErrorFormat> QT_APPLICATION_ERROR_FILTERS = QList<ErrorFormat>()
+    // ASSERT: "errors().isEmpty()" in file /foo/bar.cpp, line 49
+    << ErrorFormat("^ASSERT: \"(.*)\" in file (.*), line ([0-9]+)$", 2, 3, -1)
+    // QFATAL : FooTest::testBar() ASSERT: "index.isValid()" in file /foo/bar.cpp, line 32
+    << ErrorFormat("^QFATAL : (.*) ASSERT: \"(.*)\" in file (.*), line ([0-9]+)$", 3, 4, -1)
+    // Catch:
+    // FAIL!  : FooTest::testBar() Compared pointers are not the same
+    //    Actual   ...
+    //    Expected ...
+    //    Loc: [/foo/bar.cpp(33)]
+    //
+    // Do *not* catch:
+    //    ...
+    //    Loc: [Unknown file(0)]
+    << ErrorFormat("^   Loc: \\[(.*)\\(([1-9][0-9]*)\\)\\]$", 1, 2, -1);
 
-            // Make the item clickable if it comes with the necessary file & line number information
-            if (curErrFilter.fileGroup > 0 && curErrFilter.lineGroup > 0)
-                item.isActivatable = true;
+NativeAppErrorFilterStrategy::NativeAppErrorFilterStrategy()
+{
+}
 
-            break;
-        }
-    }
-    return item;
+FilteredItem NativeAppErrorFilterStrategy::actionInLine(const QString& line)
+{
+    return FilteredItem(line);
+}
+
+FilteredItem NativeAppErrorFilterStrategy::errorInLine(const QString& line)
+{
+    return FilteringStrategyUtils::match(QT_APPLICATION_ERROR_FILTERS, line);
 }
 
 /// --- Static Analysis filter strategy ---
@@ -378,33 +432,7 @@ FilteredItem StaticAnalysisFilterStrategy::actionInLine(const QString& line)
 
 FilteredItem StaticAnalysisFilterStrategy::errorInLine(const QString& line)
 {
-    FilteredItem item(line);
-    foreach( const ErrorFormat& curErrFilter, STATIC_ANALYSIS_FILTERS ) {
-        QRegExp regEx = curErrFilter.expression;
-        if( regEx.indexIn( line ) != -1 )
-        {
-            item.url = regEx.cap( curErrFilter.fileGroup );
-            item.lineNo = regEx.cap( curErrFilter.lineGroup ).toInt() - 1;
-            if(curErrFilter.columnGroup >= 0) {
-                item.columnNo = regEx.cap( curErrFilter.columnGroup ).toInt() - 1;
-            } else {
-                item.columnNo = 0;
-            }
-
-            QString txt = regEx.cap(curErrFilter.textGroup);
-
-            item.type = FilteredItem::ErrorItem;
-
-            // Make the item clickable if it comes with the necessary file & line number information
-            if (curErrFilter.fileGroup > 0 && curErrFilter.lineGroup > 0) {
-                item.isActivatable = true;
-            }
-            break;
-        }
-    }
-    return item;
+    return FilteringStrategyUtils::match(STATIC_ANALYSIS_FILTERS, line);
 }
 
-
-} // namespace KDevelop
-
+}
