@@ -30,6 +30,7 @@
 
 #include <interfaces/ilaunchconfiguration.h>
 #include <outputview/outputmodel.h>
+#include <outputview/outputdelegate.h>
 #include <util/processlinemaker.h>
 #include <util/environmentgrouplist.h>
 
@@ -41,6 +42,8 @@
 
 #include "iexecuteplugin.h"
 
+using namespace KDevelop;
+
 NativeAppJob::NativeAppJob(QObject* parent, KDevelop::ILaunchConfiguration* cfg) 
     : KDevelop::OutputJob( parent ), proc(0)
 {
@@ -90,8 +93,11 @@ NativeAppJob::NativeAppJob(QObject* parent, KDevelop::ILaunchConfiguration* cfg)
     
     setStandardToolView(KDevelop::IOutputView::RunView);
     setBehaviours(KDevelop::IOutputView::AllowUserClose | KDevelop::IOutputView::AutoScroll);
-    setModel( new KDevelop::OutputModel );
-    
+    OutputModel* m = new OutputModel;
+    m->setFilteringStrategy(OutputModel::NativeAppErrorFilter);
+    setModel(m);
+    setDelegate( new KDevelop::OutputDelegate );
+
     connect( lineMaker, SIGNAL(receivedStdoutLines(QStringList)), model(), SLOT(appendLines(QStringList)) );
     connect( proc, SIGNAL(error(QProcess::ProcessError)), SLOT(processError(QProcess::ProcessError)) );
     connect( proc, SIGNAL(finished(int,QProcess::ExitStatus)), SLOT(processFinished(int,QProcess::ExitStatus)) );
@@ -155,11 +161,11 @@ bool NativeAppJob::doKill()
     return true;
 }
 
-
 void NativeAppJob::processFinished( int exitCode , QProcess::ExitStatus status ) 
 {
+    connect(model(), SIGNAL(allDone()), SLOT(outputDone()));
     lineMaker->flushBuffers();
-    
+
     if (exitCode == 0 && status == QProcess::NormalExit) {
         appendLine( i18n("*** Exited normally ***") );
     } else if (status == QProcess::NormalExit) {
@@ -172,7 +178,12 @@ void NativeAppJob::processFinished( int exitCode , QProcess::ExitStatus status )
         appendLine( i18n("*** Crashed with return code: %1 ***", QString::number(exitCode)) );
         setError(OutputJob::FailedShownError);
     }
-    kDebug() << "Process done";
+
+    model()->ensureAllDone();
+}
+
+void NativeAppJob::outputDone()
+{
     emitResult();
 }
 
