@@ -23,8 +23,10 @@
 #define KDEVPLATFORM_PLUGIN_GIT_PLUGIN_H
 
 #include <vcs/interfaces/idistributedversioncontrol.h>
+#include <vcs/interfaces/icontentawareversioncontrol.h>
 #include <vcs/dvcs/dvcsplugin.h>
 #include <QObject>
+#include <QProcess>
 #include <vcs/vcsstatusinfo.h>
 #include <outputview/outputjob.h>
 #include <vcs/vcsjob.h>
@@ -64,10 +66,10 @@ class StandardJob : public KDevelop::VcsJob
  * It implements the DVCS dependent things not implemented in KDevelop::DistributedVersionControlPlugin
  * @author Evgeniy Ivanov <powerfox@kde.ru>
  */
-class GitPlugin: public KDevelop::DistributedVersionControlPlugin
+class GitPlugin: public KDevelop::DistributedVersionControlPlugin, public KDevelop::IContentAwareVersionControl
 {
     Q_OBJECT
-    Q_INTERFACES(KDevelop::IBasicVersionControl KDevelop::IDistributedVersionControl)
+    Q_INTERFACES(KDevelop::IBasicVersionControl KDevelop::IDistributedVersionControl KDevelop::IContentAwareVersionControl)
     friend class GitInitTest;
 public:
     GitPlugin(QObject *parent, const QVariantList & args = QVariantList() );
@@ -135,10 +137,13 @@ public:
     
     bool hasStashes(const QDir& repository);
     bool hasModifications(const QDir& repository);
+    bool hasModifications(const QDir& repo, const KUrl& file);
 
     virtual bool hasError() const;
     virtual QString errorDescription() const;
     virtual void registerRepositoryForCurrentBranchChanges(const KUrl& repository);
+
+    KDevelop::CheckInRepositoryJob* isInRepository(KTextEditor::Document* document);
 protected:
   
     KUrl repositoryRoot(const KUrl& path);
