@@ -31,6 +31,10 @@ public:
 private slots:
     void testSorting();
     void testSorting_data();
+    void testAbbreviations();
+    void testAbbreviations_data();
+    void testDuchainFilter();
+    void testDuchainFilter_data();
 
     void testProjectFileFilter();
 };
