@@ -44,19 +44,22 @@
 #include <ktabwidget.h>
 #include <kstandardaction.h>
 #include <klineedit.h>
+#include <kactioncollection.h>
 
 #include <outputview/ioutputviewmodel.h>
 #include <util/focusedtreeview.h>
 
 #include "toolviewdata.h"
 
+Q_DECLARE_METATYPE(QTreeView*)
+
 OutputWidget::OutputWidget(QWidget* parent, const ToolViewData* tvdata)
     : QWidget( parent )
     , tabwidget(0)
     , stackwidget(0)
     , data(tvdata)
     , m_closeButton(0)
-    , m_closeAllButton(0)
+    , m_closeOthersAction(0)
     , nextAction(0)
     , previousAction(0)
     , activateOnSelect(0)
@@ -73,25 +76,18 @@ OutputWidget::OutputWidget(QWidget* parent, const ToolViewData* tvdata)
         tabwidget = new KTabWidget(this);
         layout->addWidget( tabwidget );
         m_closeButton = new QToolButton( this );
-        connect( m_closeButton, SIGNAL(clicked()),
-                 this, SLOT(closeActiveView()) );
+        connect( m_closeButton, SIGNAL(clicked()), SLOT(closeActiveView()) );
         m_closeButton->setIcon( KIcon("tab-close") );
-        m_closeButton->adjustSize();
         m_closeButton->setToolTip( i18n( "Close the currently active output view") );
-        m_closeAllButton = new QToolButton( this );
-        connect(m_closeAllButton, SIGNAL(clicked()),
-                this, SLOT(closeOtherViews()));
-        m_closeAllButton->setIcon(KIcon("tab-close-other"));
-        m_closeAllButton->adjustSize();
-        m_closeAllButton->setToolTip( i18n( "Close all other output views" ) );
 
-        QWidget* toolbar = new QWidget(this);
-        QBoxLayout* toolbarLayout =  new QBoxLayout(QBoxLayout::LeftToRight);
-        toolbarLayout->setMargin(0);
-        toolbar->setLayout(toolbarLayout);
-        toolbarLayout->addWidget(m_closeButton);
-        toolbarLayout->addWidget(m_closeAllButton);
-        tabwidget->setCornerWidget(toolbar, Qt::TopRightCorner);
+        m_closeOthersAction = new QAction( this );
+        connect(m_closeOthersAction, SIGNAL(triggered(bool)), SLOT(closeOtherViews()));
+        m_closeOthersAction->setIcon(KIcon("tab-close-other"));
+        m_closeOthersAction->setToolTip( i18n( "Close all other output views" ) );
+        m_closeOthersAction->setText( m_closeOthersAction->toolTip() );
+        addAction(m_closeOthersAction);
+
+        tabwidget->setCornerWidget(m_closeButton, Qt::TopRightCorner);
     } else if ( data->type == KDevelop::IOutputView::HistoryView )
     {
         stackwidget = new QStackedWidget( this );
@@ -105,6 +101,9 @@ OutputWidget::OutputWidget(QWidget* parent, const ToolViewData* tvdata)
         addAction(nextAction);
     }
 
+    addAction(dynamic_cast<KAction*>(data->plugin->actionCollection()->action("prev_error")));
+    addAction(dynamic_cast<KAction*>(data->plugin->actionCollection()->action("next_error")));
+
     activateOnSelect = new KToggleAction( KIcon(), i18n("Select activated Item"), this );
     activateOnSelect->setChecked( true );
     focusOnSelect = new KToggleAction( KIcon(), i18n("Focus when selecting Item"), this );
@@ -230,12 +229,12 @@ void OutputWidget::removeOutput( int id )
 {
     if( data->outputdata.contains( id ) && views.contains( id ) )
     {
+        QTreeView* view = views.value(id);
         if( data->type & KDevelop::IOutputView::MultipleView || data->type & KDevelop::IOutputView::HistoryView )
         {
-            QTreeView* w = views.value(id);
             if( data->type & KDevelop::IOutputView::MultipleView )
             {
-                int idx = tabwidget->indexOf( w );
+                int idx = tabwidget->indexOf( view );
                 if( idx != -1 )
                 {
                     tabwidget->removeTab( idx );
@@ -247,16 +246,15 @@ void OutputWidget::removeOutput( int id )
                 }
             } else
             {
-                int idx = stackwidget->indexOf( w );
+                int idx = stackwidget->indexOf( view );
                 if( idx != -1 && proxyModels.contains( idx ) )
                 {
                     delete proxyModels.take( idx );
                     filters.remove( idx );
                 }
-                stackwidget->removeWidget( w );
+                stackwidget->removeWidget( view );
             }
-            delete w;
-            views.remove( id );
+            delete view;
         } else
         {
             views.value( id )->setModel( 0 );
@@ -268,8 +266,9 @@ void OutputWidget::removeOutput( int id )
         }
         disconnect( data->outputdata.value( id )->model,SIGNAL(rowsInserted(QModelIndex,int,int)),
                     this, SLOT(rowsInserted(QModelIndex,int,int)) );
-        
+
         views.remove( id );
+        m_scrollDelay.remove( view );
         emit outputRemoved( data->toolViewId, id );
     }
     enableActions();
@@ -313,7 +312,7 @@ void OutputWidget::closeOtherViews()
     enableActions();
 }
 
-QWidget* OutputWidget::currentWidget()
+QWidget* OutputWidget::currentWidget() const
 {
     QWidget* widget;
     if( data->type & KDevelop::IOutputView::MultipleView )
@@ -329,87 +328,94 @@ QWidget* OutputWidget::currentWidget()
     return widget;
 }
 
-void OutputWidget::selectNextItem()
+KDevelop::IOutputViewModel *OutputWidget::outputViewModel() const
 {
     QWidget* widget = currentWidget();
 
     if( !widget || !widget->isVisible() )
-        return;
+        return nullptr;
 
-    if( focusOnSelect->isChecked() && !widget->hasFocus() )
-    {
-        widget->setFocus( Qt::OtherFocusReason );
-    }
-
-    QAbstractItemView *view = dynamic_cast<QAbstractItemView*>(widget);
+    auto view = qobject_cast<QAbstractItemView*>(widget);
     if( !view )
-        return;
+        return nullptr;
 
     QAbstractItemModel *absmodel = view->model();
     KDevelop::IOutputViewModel *iface = dynamic_cast<KDevelop::IOutputViewModel*>(absmodel);
-    if( iface )
+    if ( ! iface )
     {
-        kDebug() << "selecting next item";
-        QModelIndex index = iface->nextHighlightIndex( view->currentIndex() );
-        if( index.isValid() )
+        // try if it's a proxy model?
+        if ( QAbstractProxyModel* proxy = qobject_cast<QAbstractProxyModel*>(absmodel) )
         {
-            view->setCurrentIndex( index );
-            view->scrollTo( index );
-            if( activateOnSelect->isChecked() )
-            {
-                iface->activate( index );
-            }
+            iface = dynamic_cast<KDevelop::IOutputViewModel*>(proxy->sourceModel());
         }
     }
+    return iface;
+}
+
+void OutputWidget::eventuallyDoFocus()
+{
+    QWidget* widget = currentWidget();
+    if( focusOnSelect->isChecked() && !widget->hasFocus() ) {
+        widget->setFocus( Qt::OtherFocusReason );
+    }
+}
+
+QAbstractItemView *OutputWidget::outputView() const
+{
+    auto widget = currentWidget();
+    return qobject_cast<QAbstractItemView*>(widget);
+}
+
+void OutputWidget::activateIndex(const QModelIndex &index, QAbstractItemView *view, KDevelop::IOutputViewModel *iface)
+{
+    if( ! index.isValid() )
+        return;
+    view->setCurrentIndex( index );
+    view->scrollTo( index );
+    int tabIndex = currentOutputIndex();
+    QModelIndex mapped = index;
+    if( QAbstractProxyModel* proxy = proxyModels.value(tabIndex) ) {
+        if ( index.model() == proxy ) {
+            mapped = proxy->mapToSource(index);
+        }
+    }
+    if( activateOnSelect->isChecked() )
+    {
+        iface->activate( mapped );
+    }
+}
+
+void OutputWidget::selectNextItem()
+{
+    auto view = outputView();
+    auto iface = outputViewModel();
+    if ( ! view || ! iface )
+        return;
+    eventuallyDoFocus();
+    kDebug() << "selecting next item";
+    QModelIndex index = iface->nextHighlightIndex( view->currentIndex() );
+    activateIndex(index, view, iface);
 }
 
 void OutputWidget::selectPrevItem()
 {
-    QWidget* widget = currentWidget();
-    if( !widget || !widget->isVisible() )
+    auto view = outputView();
+    auto iface = outputViewModel();
+    if ( ! view || ! iface )
         return;
-    QAbstractItemView *view = dynamic_cast<QAbstractItemView*>(widget);
-    if( !view )
-        return;
-
-    if( focusOnSelect->isChecked() && !widget->hasFocus() )
-    {
-        widget->setFocus( Qt::OtherFocusReason );
-    }
-
-    QAbstractItemModel *absmodel = view->model();
-    KDevelop::IOutputViewModel *iface = dynamic_cast<KDevelop::IOutputViewModel*>(absmodel);
-    if( iface )
-    {
-        kDebug() << "activating previous item";
-        QModelIndex index = iface->previousHighlightIndex( view->currentIndex() );
-        if( index.isValid() )
-        {
-            view->setCurrentIndex( index );
-            view->scrollTo( index );
-            if( activateOnSelect->isChecked() )
-            {
-                iface->activate( index );
-            }
-        }
-    }
+    eventuallyDoFocus();
+    kDebug() << "activating previous item";
+    QModelIndex index = iface->previousHighlightIndex( view->currentIndex() );
+    activateIndex(index, view, iface);
 }
 
 void OutputWidget::activate(const QModelIndex& index)
 {
-    QWidget* widget = currentWidget();
-    if( !widget )
+    auto iface = outputViewModel();
+    auto view = outputView();
+    if( ! view || ! iface )
         return;
-    QAbstractItemView *view = dynamic_cast<QAbstractItemView*>(widget);
-    if( !view )
-        return;
-
-    QAbstractItemModel *absmodel = view->model();
-    KDevelop::IOutputViewModel *iface = dynamic_cast<KDevelop::IOutputViewModel*>(absmodel);
-    if( iface )
-    {
-        iface->activate( index );
-    }
+    activateIndex(index, view, iface);
 }
 
 static QTreeView* createFocusedTreeView( QWidget* parent )
@@ -428,6 +434,7 @@ static QTreeView* createFocusedTreeView( QWidget* parent )
 QTreeView* OutputWidget::createListView(int id)
 {
     QTreeView* listview = 0;
+    bool newView = true;
     if( !views.contains(id) )
     {
         if( data->type & KDevelop::IOutputView::MultipleView || data->type & KDevelop::IOutputView::HistoryView )
@@ -463,9 +470,20 @@ QTreeView* OutputWidget::createListView(int id)
             } else
             {
                 listview = views.begin().value();
+                newView = false;
             }
             views[id] = listview;
         }
+
+        if (newView) {
+            auto timer = new QTimer(listview);
+            timer->setSingleShot(true);
+            timer->setInterval(300);
+            timer->setProperty("view", QVariant::fromValue(listview));
+            m_scrollDelay[listview] = {timer, -1, -1};
+            connect(timer, SIGNAL(timeout()), SLOT(delayedScroll()));
+        }
+
         changeModel( id );
         changeDelegate( id );
     } else
@@ -530,17 +548,53 @@ void OutputWidget::enableActions()
 }
 
 void OutputWidget::rowsInserted(const QModelIndex& parent, int from, int to) {
-    Q_UNUSED(parent);
-    for( QMap< int, QTreeView* >::const_iterator it = views.constBegin(); it != views.constEnd(); ++it) {
-        if((*it)->model() == sender()) {
-            QModelIndex pre = (*it)->model()->index(from-1, 0);
-            if(!pre.isValid() || ((*it)->visualRect(pre).isValid() && (*it)->viewport()->rect().intersects((*it)->visualRect(pre)) && to == (*it)->model()->rowCount()-1)) {
-                (*it)->scrollToBottom();
+    if (parent.isValid()) {
+        return;
+    }
+
+    auto model = qobject_cast<QAbstractItemModel*>(sender());
+    Q_ASSERT(model);
+
+    foreach (QTreeView* view, views) {
+        if (view->model() == model) {
+            auto data = m_scrollDelay[view];
+            if (data.from == -1) {
+                data.from = from;
+            }
+            data.to = to;
+            if (!data.timer->isActive()) {
+                data.timer->start();
             }
         }
     }
 }
 
+void OutputWidget::delayedScroll()
+{
+    auto timer = qobject_cast<QTimer*>(sender());
+    Q_ASSERT(timer);
+    auto view = timer->property("view").value<QTreeView*>();
+    Q_ASSERT(view);
+    delayedScroll(view);
+}
+
+void OutputWidget::delayedScroll(QTreeView* view)
+{
+    auto& data = m_scrollDelay[view];
+
+    QModelIndex pre = view->model()->index(data.from - 1, 0);
+    bool scroll = !pre.isValid();
+    if (!scroll && data.to == view->model()->rowCount() - 1) {
+        auto rect = view->visualRect(pre);
+        scroll = rect.isValid() && view->viewport()->rect().intersects(rect);
+    }
+    if (scroll) {
+        view->scrollToBottom();
+    }
+
+    data.from = -1;
+}
+
 void OutputWidget::scrollToIndex( const QModelIndex& idx )
 {
     QWidget* w = currentWidget();
@@ -580,14 +634,8 @@ void OutputWidget::selectAll()
     view->selectAll();
 }
 
-void OutputWidget::outputFilter(const QString filter)
+int OutputWidget::currentOutputIndex()
 {
-    QWidget* widget = currentWidget();
-    if( !widget )
-        return;
-    QAbstractItemView *view = dynamic_cast<QAbstractItemView*>(widget);
-    if( !view )
-        return;
     int index = 0;
     if( data->type & KDevelop::IOutputView::MultipleView )
     {
@@ -596,9 +644,21 @@ void OutputWidget::outputFilter(const QString filter)
     {
         index = stackwidget->currentIndex();
     }
+    return index;
+}
+
+void OutputWidget::outputFilter(const QString filter)
+{
+    QWidget* widget = currentWidget();
+    if( !widget )
+        return;
+    QAbstractItemView *view = dynamic_cast<QAbstractItemView*>(widget);
+    if( !view )
+        return;
+    int index = currentOutputIndex();
     if( !dynamic_cast<QSortFilterProxyModel*>(view->model()) )
     {
-        QSortFilterProxyModel* _proxyModel = new QSortFilterProxyModel(view->model());
+         QSortFilterProxyModel* _proxyModel = new QSortFilterProxyModel(view->model());
         _proxyModel->setDynamicSortFilter(true);
         _proxyModel->setSourceModel(view->model());
         proxyModels.insert(index, _proxyModel);
@@ -620,4 +680,11 @@ void OutputWidget::updateFilter(int index)
     }
 }
 
+void OutputWidget::setTitle(int outputId, const QString& title)
+{
+    if( data->type & KDevelop::IOutputView::MultipleView ) {
+        tabwidget->setTabText(outputId - 1, title);
+    }
+}
+
 #include "outputwidget.moc"
