@@ -23,13 +23,19 @@ Boston, MA 02110-1301, USA.
 #include <kconfiggroup.h>
 #include <kapplication.h>
 #include <kaboutapplicationdialog.h>
+#include <knotifyconfigwidget.h>
+#include <ktogglefullscreenaction.h>
 
 #include <ktexteditor/editor.h>
 
+#include <sublime/area.h>
+#include <sublime/view.h>
+
 #include "core.h"
 #include "documentcontroller.h"
 #include "mainwindow_p.h"
 #include "sessiondialog.h"
+#include "uicontroller.h"
 
 #include "mainwindow.h"
 #include "loadedpluginsdialog.h"
@@ -77,25 +83,11 @@ void MainWindowPrivate::gotoPreviousWindow()
     gotoPrevNextWindow(false);
 }
 
-void MainWindowPrivate::configureToolbars()
-{}
-
 void MainWindowPrivate::newToolbarConfig()
 {
     m_mainWindow->applyMainWindowSettings( KConfigGroup(KGlobal::config(), "MainWindow") );
 }
 
-void MainWindowPrivate::stopButtonPressed()
-{}
-
-void MainWindowPrivate::stopPopupActivated( int id )
-{
-    Q_UNUSED( id );
-}
-
-void MainWindowPrivate::stopMenuAboutToShow()
-{}
-
 void MainWindowPrivate::settingsDialog()
 {
     Core::self()->uiControllerInternal()->showSettingsDialog();
@@ -130,6 +122,45 @@ void MainWindowPrivate::split(Qt::Orientation orientation)
     m_mainWindow->activateView(newView);
 }
 
+static void gotoPrevNextSplit(bool next)
+{
+    UiController* ui = Core::self()->uiControllerInternal();
+
+    if( !ui->activeSublimeWindow() )
+        return;
+
+    Sublime::Area* area = ui->activeSublimeWindow()->area();
+    if (!area)
+        return;
+
+    QList<Sublime::View*> topViews = ui->activeSublimeWindow()->getTopViews();
+
+    Sublime::View *activeView = ui->activeSublimeWindow()->activeView();
+    if (!activeView)
+        return;
+
+    int viewIndex = topViews.indexOf(activeView);
+    viewIndex = next ? viewIndex + 1 : viewIndex -1;
+
+    if (viewIndex < 0)
+        viewIndex = topViews.count() - 1;
+    else if (viewIndex >= topViews.count())
+        viewIndex = 0;
+
+    if (viewIndex >= 0 && viewIndex < topViews.count())
+        ui->activeSublimeWindow()->activateView(topViews.at(viewIndex));
+}
+
+void MainWindowPrivate::gotoNextSplit()
+{
+    gotoPrevNextSplit(true);
+}
+
+void MainWindowPrivate::gotoPreviousSplit()
+{
+    gotoPrevNextSplit(false);
+}
+
 void MainWindowPrivate::toggleFullScreen(bool fullScreen)
 {
     KToggleFullScreenAction::setFullScreen( m_mainWindow, fullScreen );
