@@ -112,6 +112,7 @@ public:
     bool m_foundProjectFile; //Temporary flag used while searching the hierarchy for a project file
     bool m_cleaningUp; //Temporary flag enabled while destroying the project-controller
     QPointer<ProjectChangesModel> m_changesModel;
+    QHash< IProject*, QPointer<KJob> > m_parseJobs; // parse jobs that add files from the project to the background parser.
 
     ProjectControllerPrivate( ProjectController* p )
         : m_core(0), model(0), selectionModel(0), dialog(0), m_configuringProject(0), q(p), buildset(0), m_foundProjectFile(false), m_cleaningUp(false)
@@ -143,8 +144,8 @@ public:
         cfgDlg.setKCMArguments( QStringList()
                                     << proj->developerTempFile()
                                     << proj->projectTempFile()
-                                    << proj->projectFileUrl().url()
-                                    << proj->developerFileUrl().url()
+                                    << proj->projectFile().pathOrUrl()
+                                    << proj->developerFile().pathOrUrl()
                                     << proj->name() );
         m_configuringProject = proj;
         cfgDlg.setWindowTitle( i18n("Configure Project %1", proj->name()) );
@@ -152,19 +153,17 @@ public:
         proj->projectConfiguration()->sync();
         m_configuringProject = 0;
     }
+
     void saveListOfOpenedProjects()
     {
-        KSharedConfig::Ptr config = Core::self()->activeSession()->config();
-        KConfigGroup group = config->group( "General Options" );
-    
         KUrl::List openProjects;
-    
+        openProjects.reserve( m_projects.size() );
+
         foreach( IProject* project, m_projects ) {
-            openProjects.append(project->projectFileUrl());
+            openProjects.append(project->projectFile().toUrl());
         }
-    
-        group.writeEntry( "Open Projects", openProjects.toStringList() );
-        group.sync();
+
+        Core::self()->activeSession()->setContainedProjects( openProjects );
     }
 
     // Recursively collects builder dependencies for a project.
@@ -303,13 +302,19 @@ public:
         Project* project = new Project();
         QObject::connect(project, SIGNAL(aboutToOpen(KDevelop::IProject*)),
                          q, SIGNAL(projectAboutToBeOpened(KDevelop::IProject*)));
-        if ( !project->open( url ) )
+        if ( !project->open( Path(url) ) )
         {
             m_currentlyOpening.removeAll(url);
             q->abortOpeningProject(project);
             project->deleteLater();
         }
     }
+
+    void areaChanged(Sublime::Area* area) {
+        KActionCollection* ac = m_core->uiControllerInternal()->defaultMainWindow()->actionCollection();
+        ac->action("commit_current_project")->setEnabled(area->objectName() == "code");
+        ac->action("commit_current_project")->setVisible(area->objectName() == "code");
+    }
 };
 
 IProjectDialogProvider::IProjectDialogProvider()
@@ -527,6 +532,8 @@ void ProjectController::setupActions()
     action->setText( i18n( "Commit Current Project..." ) );
     action->setIconText( i18n( "Commit..." ) );
     action->setIcon( KIcon("svn-commit") );
+    connect(d->m_core->uiControllerInternal()->defaultMainWindow(), SIGNAL(areaChanged(Sublime::Area*)),
+            SLOT(areaChanged(Sublime::Area*)));
     d->m_core->uiControllerInternal()->area(0, "code")->addAction(action);
 
     KSharedConfig * config = KGlobal::config().data();
@@ -553,6 +560,10 @@ ProjectController::~ProjectController()
 
 void ProjectController::cleanup()
 {
+    if ( d->m_currentlyOpening.isEmpty() ) {
+        d->saveListOfOpenedProjects();
+    }
+
     d->m_cleaningUp = true;
     if( buildSetModel() ) {
         buildSetModel()->storeToSession( Core::self()->activeSession() );
@@ -792,14 +803,7 @@ void ProjectController::projectImportingFinished( IProject* project )
     d->m_projectPlugins.insert( project, pluglist );
     d->m_projects.append( project );
 
-    if(!Core::self()->sessionController()->activeSession()->containedProjects().contains(project->projectFileUrl()))
-        d->saveListOfOpenedProjects();
-    
-//     KActionCollection * ac = d->m_core->uiController()->defaultMainWindow()->actionCollection();
-//     QAction * action;
-
-    //action = ac->action( "project_close" );
-    //action->setEnabled( true );
+    d->saveListOfOpenedProjects();
 
     if (Core::self()->setupFlags() != Core::NoUi)
     {
@@ -817,8 +821,7 @@ void ProjectController::projectImportingFinished( IProject* project )
 
     if (parseAllProjectSources())
     {
-        KJob* parseProjectJob = new KDevelop::ParseProjectJob(project);
-        ICore::self()->runController()->registerJob(parseProjectJob);
+        reparseProject(project);
     }
 
     // Add all currently open files that belong to the project to the background-parser,
@@ -863,9 +866,10 @@ void ProjectController::unloadUnusedProjectPlugins(IProject* proj)
 // helper method for closeProject()
 void ProjectController::closeAllOpenedFiles(IProject* proj)
 {
-    Q_FOREACH( ProjectFileItem *fileItem, proj->files() )
-    {
-        Core::self()->documentControllerInternal()->closeDocument( fileItem->url() );
+    foreach(IDocument* doc, Core::self()->documentController()->openDocuments()) {
+        if (proj->inProject(IndexedString(doc->url()))) {
+            doc->close();
+        }
     }
 }
 
@@ -928,10 +932,13 @@ ProjectModel* ProjectController::projectModel()
 
 IProject* ProjectController::findProjectForUrl( const KUrl& url ) const
 {
-    Q_FOREACH( IProject* proj, d->m_projects )
-    {
-        if( proj->inProject( url ) )
-            return proj;
+    if (d->m_projects.isEmpty()) {
+        return 0;
+    }
+
+    ProjectBaseItem* item = d->model->itemForPath(IndexedString(url));
+    if (item) {
+        return item->project();
     }
     return 0;
 }
@@ -983,7 +990,7 @@ KUrl ProjectController::projectsBaseDirectory() const
                                      KUrl( QDir::homePath()+"/projects" ) );
 }
 
-QString ProjectController::prettyFilePath(KUrl url, FormattingOptions format) const
+QString ProjectController::prettyFilePath(const KUrl& url, FormattingOptions format) const
 {
     IProject* project = Core::self()->projectController()->findProjectForUrl(url);
     
@@ -1007,7 +1014,7 @@ QString ProjectController::prettyFilePath(KUrl url, FormattingOptions format) co
         } else {
             prefixText = project->name() + '/';
         }
-        QString relativePath = project->relativeUrl(url.upUrl()).path(KUrl::AddTrailingSlash);
+        QString relativePath = KUrl::relativeUrl(project->folder(), url.upUrl());
         if(relativePath.startsWith("./"))
             relativePath = relativePath.mid(2);
         prefixText += relativePath;
@@ -1015,7 +1022,7 @@ QString ProjectController::prettyFilePath(KUrl url, FormattingOptions format) co
     return prefixText;
 }
 
-QString ProjectController::prettyFileName(KUrl url, FormattingOptions format) const
+QString ProjectController::prettyFileName(const KUrl& url, FormattingOptions format) const
 {
     IProject* project = Core::self()->projectController()->findProjectForUrl(url);
     if(project && project->folder().equals(url, KUrl::CompareWithoutTrailingSlash))
@@ -1070,14 +1077,14 @@ void ProjectController::commitCurrentProject()
     IProject* project = ICore::self()->projectController()->findProjectForUrl(url);
     
     if(project && project->versionControlPlugin()) {
-        KUrl baseUrl=project->projectItem()->url();
         IPlugin* plugin = project->versionControlPlugin();
         IBasicVersionControl* vcs=plugin->extension<IBasicVersionControl>();
         
         if(vcs) {
             ICore::self()->documentController()->saveAllDocuments(KDevelop::IDocument::Silent);
 
-            VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(new VCSStandardDiffUpdater(vcs, baseUrl));
+            const Path basePath = project->path();
+            VCSCommitDiffPatchSource* patchSource = new VCSCommitDiffPatchSource(new VCSStandardDiffUpdater(vcs, basePath.toUrl()));
 
             bool ret = showVcsDiff(patchSource);
 
@@ -1090,18 +1097,18 @@ void ProjectController::commitCurrentProject()
     }
 }
 
-QString ProjectController::mapSourceBuild( const QString& path, bool reverse, bool fallbackRoot ) const
+QString ProjectController::mapSourceBuild( const QString& path_, bool reverse, bool fallbackRoot ) const
 {
-    KUrl url(path);
+    Path path(path_);
     IProject* sourceDirProject = 0, *buildDirProject = 0;
     Q_FOREACH(IProject* proj, d->m_projects)
     {
-        if(proj->folder().isParentOf(url))
+        if(proj->path().isParentOf(path))
             sourceDirProject = proj;
         if(proj->buildSystemManager())
         {
-            KUrl buildDir = proj->buildSystemManager()->buildDirectory(proj->projectItem());
-            if(buildDir.isValid() && buildDir.isParentOf(url))
+            Path buildDir = proj->buildSystemManager()->buildDirectory(proj->projectItem());
+            if(buildDir.isValid() && buildDir.isParentOf(path))
                 buildDirProject = proj;
         }
     }
@@ -1112,12 +1119,12 @@ QString ProjectController::mapSourceBuild( const QString& path, bool reverse, bo
         if(sourceDirProject && sourceDirProject->buildSystemManager())
         {
             // We're in the source, map into the build directory
-            QString relativePath = KUrl::relativeUrl(sourceDirProject->folder(), url);
+            QString relativePath = sourceDirProject->path().relativePath(path);
             
-            KUrl build = sourceDirProject->buildSystemManager()->buildDirectory(sourceDirProject->projectItem());
+            Path build = sourceDirProject->buildSystemManager()->buildDirectory(sourceDirProject->projectItem());
             build.addPath(relativePath);
             while(!QFile::exists(build.path()))
-                build = build.upUrl();
+                build = build.parent();
             return build.pathOrUrl();
         }else if(buildDirProject && fallbackRoot)
         {
@@ -1128,24 +1135,34 @@ QString ProjectController::mapSourceBuild( const QString& path, bool reverse, bo
         // Map-target is the source directory
         if(buildDirProject)
         {
-            KUrl build = buildDirProject->buildSystemManager()->buildDirectory(buildDirProject->projectItem());
+            Path build = buildDirProject->buildSystemManager()->buildDirectory(buildDirProject->projectItem());
             // We're in the source, map into the build directory
-            QString relativePath = KUrl::relativeUrl(build, url);
+            QString relativePath = build.relativePath(path);
             
-            KUrl source = buildDirProject->folder();
+            Path source = buildDirProject->path();
             source.addPath(relativePath);
             while(!QFile::exists(source.path()))
-                source = source.upUrl();
+                source = source.parent();
             return source.pathOrUrl();
         }else if(sourceDirProject && fallbackRoot)
         {
             // We're in the source directory, map to the root
-            return sourceDirProject->folder().pathOrUrl();
+            return sourceDirProject->path().pathOrUrl();
         }
     }
     return QString();
 }
 
+void ProjectController::reparseProject( IProject* project, bool forceUpdate )
+{
+    if (auto job = d->m_parseJobs.value(project)) {
+        job->kill();
+    }
+
+    d->m_parseJobs[project] = new KDevelop::ParseProjectJob(project, forceUpdate);
+    ICore::self()->runController()->registerJob(d->m_parseJobs[project]);
+}
+
 }
 
 #include "projectcontroller.moc"
