@@ -21,6 +21,7 @@
 #include <QMap>
 #include <QAction>
 #include <QPointer>
+#include <QTimer>
 #include <QApplication>
 #include <QListWidget>
 #include <QToolBar>
@@ -38,6 +39,7 @@
 #include <sublime/area.h>
 #include <sublime/view.h>
 #include <sublime/tooldocument.h>
+#include <sublime/holdupdates.h>
 
 #include "core.h"
 #include "shellextension.h"
@@ -111,6 +113,9 @@ public:
         {
             activeSublimeWindow = defaultMainWindow = 0;
         }
+
+        m_assistantTimer.setSingleShot(true);
+        m_assistantTimer.setInterval(100);
     }
 
     void widgetChanged(QWidget*, QWidget* now)
@@ -134,6 +139,7 @@ public:
 
     //Currently shown assistant popup.
     QPointer<AssistantPopup> currentShownAssistant;
+    QTimer m_assistantTimer;
 
 private:
     UiController *m_controller;
@@ -489,11 +495,8 @@ void UiController::saveAllAreas(KSharedConfig::Ptr config)
     uiConfig.writeEntry("Main Windows Count", wc);
     for (int w = 0; w < wc; ++w)
     {
-        Sublime::MainWindow *mw = mainWindows()[w];
-
         KConfigGroup mainWindowConfig(&uiConfig,
                                       QString("Main Window %1").arg(w));
-        mainWindowConfig.writeEntry("currentArea", mw->area()->objectName());
 
         foreach (Sublime::Area* defaultArea, defaultAreas())
         {
@@ -539,10 +542,6 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
     {
         KConfigGroup mainWindowConfig(&uiConfig,
                                       QString("Main Window %1").arg(w));
-        QString currentArea = mainWindowConfig.readEntry("currentArea", "");
-        
-        if(currentArea == "test") ///@todo The area was renamed, and it will lead to a crash, so this is a dirty temporary workaround
-            currentArea = "review";
         
         Sublime::MainWindow *mw = mainWindows()[w];
 
@@ -580,12 +579,8 @@ void UiController::loadAllAreas(KSharedConfig::Ptr config)
             }
         }
 
-        // FIXME: check that an area of this name exists.
-        if (!currentArea.isEmpty())
-            showArea(currentArea, mw);
-        else
-            // Force reload of the changes.
-            showAreaInternal(mw->area(), mw);
+        // Force reload of the changes.
+        showAreaInternal(mw->area(), mw);
 
         mw->enableAreaSettingsSave();
     }
@@ -651,25 +646,22 @@ void UiController::showErrorMessage(const QString& message, int timeout)
     QMetaObject::invokeMethod(mw, "showErrorMessage", Q_ARG(QString, message), Q_ARG(int, timeout));
 }
 
-void UiController::hideAssistant(const KDevelop::IAssistant::Ptr& assistant)
+void UiController::hideAssistant()
 {
-    if(d->currentShownAssistant && d->currentShownAssistant->assistant() == assistant) {
+    if(d->currentShownAssistant) {
         disconnect(d->currentShownAssistant->assistant().data(), SIGNAL(hide()), this, SLOT(assistantHide()));
         disconnect(d->currentShownAssistant->assistant().data(), SIGNAL(actionsChanged()), this, SLOT(assistantActionsChanged()));
         
-        AssistantPopup* oldPopup = d->currentShownAssistant;
-        d->currentShownAssistant = 0;
-        oldPopup->hide();
-        oldPopup->deleteLater(); //We have to do deleteLater, so we don't get problems when an assistant hides itself
+        Sublime::HoldUpdates hold(ICore::self()->uiController()->activeMainWindow());
+        d->currentShownAssistant->hide();
     }
 }
 
 void UiController::popUpAssistant(const KDevelop::IAssistant::Ptr& assistant)
 {
-    assistantHide();
     if(!assistant)
         return;
-    
+
     Sublime::View* view = d->activeSublimeWindow->activeView();
     if( !view )
     {
@@ -677,14 +669,27 @@ void UiController::popUpAssistant(const KDevelop::IAssistant::Ptr& assistant)
         return;
     }
     TextEditorWidget* textWidget = dynamic_cast<TextEditorWidget*>(view->widget());
+    bool wasVisible = d->currentShownAssistant && d->currentShownAssistant->isVisible();
     if(textWidget && textWidget->editorView()) {
-
-        d->currentShownAssistant = new AssistantPopup(textWidget->editorView(), assistant);
-        if(assistant->actions().count())
+        if ( ! d->currentShownAssistant ) {
+            d->currentShownAssistant = new AssistantPopup(textWidget->editorView(), assistant);
+        } else {
+            d->currentShownAssistant->reset(textWidget->editorView(), assistant);
+        }
+        if(assistant->actions().count() && !wasVisible) {
+            Sublime::HoldUpdates hold(ICore::self()->uiController()->activeMainWindow());
+            // stop any pending hide actions
+            d->m_assistantTimer.stop();
             d->currentShownAssistant->show();
+        }
 
-        connect(assistant.data(), SIGNAL(hide()), SLOT(assistantHide()), Qt::DirectConnection);
-        connect(assistant.data(), SIGNAL(actionsChanged()), SLOT(assistantActionsChanged()), Qt::DirectConnection);
+        connect(assistant.data(), SIGNAL(hide()), SLOT(assistantHide()), Qt::UniqueConnection);
+        connect(assistant.data(), SIGNAL(actionsChanged()), SLOT(assistantActionsChanged()), Qt::UniqueConnection);
+    }
+    if( d->currentShownAssistant ) {
+        // when the assistant was still open, avoid flickering
+        d->currentShownAssistant->notifyReopened(wasVisible);
+        d->m_assistantTimer.stop();
     }
 }
 
@@ -694,13 +699,16 @@ const QMap< IToolViewFactory*, Sublime::ToolDocument* >& UiController::factoryDo
 }
 
 void UiController::assistantHide() {
-    if(d->currentShownAssistant)
-        hideAssistant(d->currentShownAssistant->assistant());
+    if(d->currentShownAssistant) {
+        connect(&d->m_assistantTimer, SIGNAL(timeout()), SLOT(hideAssistant()), Qt::UniqueConnection);
+        d->m_assistantTimer.start();
+    }
 }
 
 void UiController::assistantActionsChanged() {
-    if(d->currentShownAssistant)
+    if(d->currentShownAssistant) {
         popUpAssistant(d->currentShownAssistant->assistant());
+    }
 }
 
 }
